@@ -568,20 +568,15 @@ class Grad_2d(GradBasic):
         if not _args:
             return
 
-        u = _args[0]
-
-        if isinstance(u, Tuple):
-            n = len(u)
-            lines = []
-            for i in range(0, n):
-                line = [dx(u)[0,i], dy(u)[0,i]]
-                lines.append(line)
-
-            v = Matrix(lines)
+        u  = _args[0]
+        du = (dx(u), dy(u))
 
+        if isinstance(du[0], (Tuple, Matrix, ImmutableDenseMatrix)):
+            lines = [list(d[:]) for d in du]
         else:
-            v = Matrix((dx(u), dy(u)))
+            lines = [[d] for d in du]
 
+        v = ImmutableDenseMatrix(lines)
         return v
 
 class Grad_3d(GradBasic):
@@ -593,19 +588,15 @@ class Grad_3d(GradBasic):
         if not _args:
             return
 
-        u = _args[0]
-
-        if isinstance(u, Tuple):
-            n = len(u)
-            lines = []
-            for i in range(0, n):
-                line = [dx(u)[0,i], dy(u)[0,i], dz(u)[0,i]]
-                lines.append(line)
-
-            v = Matrix(lines)
+        u  = _args[0]
+        du = (dx(u), dy(u), dz(u))
 
+        if isinstance(du[0], (Tuple, Matrix, ImmutableDenseMatrix)):
+            lines = [list(d[:]) for d in du]
         else:
-            v = Matrix((dx(u), dy(u), dz(u)))
+            lines = [[d] for d in du]
+
+        v = ImmutableDenseMatrix(lines)
 
         return v
 
@@ -657,9 +648,9 @@ class Curl_3d(CurlBasic):
 
         u = _args[0]
 
-        return Matrix((dy(u[2]) - dz(u[1]),
-                     dz(u[0]) - dx(u[2]),
-                     dx(u[1]) - dy(u[0])))
+        return ImmutableDenseMatrix([[dy(u[2]) - dz(u[1])],
+                                     [dz(u[0]) - dx(u[2])],
+                                     [dx(u[1]) - dy(u[0])]])
 
 #==============================================================================
 class Rot_2d(CalculusFunction):
@@ -695,7 +686,7 @@ class Rot_2d(CalculusFunction):
 
         u = _args[0]
 
-        return Matrix([[dy(u),-dx(u)]]).T
+        return ImmutableDenseMatrix([[dy(u)],[-dx(u)]])
 
 #==============================================================================
 class DivBasic(CalculusFunction):
@@ -878,7 +869,7 @@ class Hessian_2d(HessianBasic):
         if isinstance(u, (VectorTestFunction, VectorField)):
             raise NotImplementedError('TODO')
 
-        return Matrix([[dx(dx(u)), dx(dy(u))],
+        return ImmutableDenseMatrix([[dx(dx(u)), dx(dy(u))],
                        [dx(dy(u)), dy(dy(u))]])
 
 class Hessian_3d(HessianBasic):
@@ -893,7 +884,7 @@ class Hessian_3d(HessianBasic):
         if isinstance(u, (VectorTestFunction, VectorField)):
             raise NotImplementedError('TODO')
 
-        return Matrix([[dx(dx(u)), dx(dy(u)), dx(dz(u))],
+        return ImmutableDenseMatrix([[dx(dx(u)), dx(dy(u)), dx(dz(u))],
                        [dx(dy(u)), dy(dy(u)), dy(dz(u))],
                        [dx(dz(u)), dy(dz(u)), dz(dz(u))]])
 
@@ -951,19 +942,15 @@ class LogicalGrad_2d(GradBasic):
         if not _args:
             return
 
-        u = _args[0]
-
-        if isinstance(u, (Tuple, Matrix, ImmutableDenseMatrix)):
-            n = len(u)
-            lines = []
-            for i in range(0, n):
-                line = [dx1(u)[0,i], dx2(u)[0,i]]
-                lines.append(line)
-            v = ImmutableDenseMatrix(lines)
+        u  = _args[0]
+        du = (dx1(u), dx2(u))
 
+        if isinstance(du[0], (Tuple, Matrix, ImmutableDenseMatrix)):
+            lines = [list(d[:]) for d in du]
         else:
-            v = ImmutableDenseMatrix((dx1(u), dx2(u)))
+            lines = [[d] for d in du]
 
+        v = ImmutableDenseMatrix(lines)
         return v
 
 class LogicalGrad_3d(GradBasic):
@@ -974,20 +961,15 @@ class LogicalGrad_3d(GradBasic):
         if not _args:
             return
 
-        u = _args[0]
-
-        if isinstance(u, (Tuple, Matrix, ImmutableDenseMatrix)):
-            n = len(u)
-            lines = []
-            for i in range(0, n):
-                line = [dx1(u)[0,i], dx2(u)[0,i], dx3(u)[0,i]]
-                lines.append(line)
-
-            v = ImmutableDenseMatrix(lines)
+        u  = _args[0]
+        du = (dx1(u), dx2(u), dx3(u))
 
+        if isinstance(du[0], (Tuple, Matrix, ImmutableDenseMatrix)):
+            lines = [list(d[:]) for d in du]
         else:
-            v = ImmutableDenseMatrix((dx1(u), dx2(u), dx3(u)))
+            lines = [[d] for d in du]
 
+        v = ImmutableDenseMatrix(lines)
         return v
 
 #==============================================================================
@@ -1014,9 +996,9 @@ class LogicalCurl_3d(CurlBasic):
 
         u = _args[0]
 
-        return ImmutableDenseMatrix((dx2(u[2]) - dx3(u[1]),
-                     dx3(u[0]) - dx1(u[2]),
-                     dx1(u[1]) - dx2(u[0])))
+        return ImmutableDenseMatrix([[dx2(u[2]) - dx3(u[1])],
+                     [dx3(u[0]) - dx1(u[2])],
+                     [dx1(u[1]) - dx2(u[0])]])
 
 #==============================================================================
 class LogicalRot_2d(CalculusFunction):
@@ -1052,7 +1034,7 @@ class LogicalRot_2d(CalculusFunction):
 
         u = _args[0]
 
-        return ImmutableDenseMatrix([[dx2(u),-dx1(u)]]).T
+        return ImmutableDenseMatrix([[dx2(u)],[-dx1(u)]])
 
 #==============================================================================
 class LogicalDiv_1d(DivBasic):
@@ -1162,7 +1144,7 @@ class LogicalHessian_2d(HessianBasic):
             raise NotImplementedError('TODO')
 
         return ImmutableDenseMatrix([[dx1(dx1(u)), dx1(dx2(u))],
-                       [dx1(dx2(u)), dx2(dx2(u))]])
+                                     [dx1(dx2(u)), dx2(dx2(u))]])
 
 class LogicalHessian_3d(HessianBasic):
 
@@ -1177,8 +1159,8 @@ class LogicalHessian_3d(HessianBasic):
             raise NotImplementedError('TODO')
 
         return ImmutableDenseMatrix([[dx1(dx1(u)), dx1(dx2(u)), dx1(dx3(u))],
-                       [dx1(dx2(u)), dx2(dx2(u)), dx2(dx3(u))],
-                       [dx1(dx3(u)), dx2(dx3(u)), dx3(dx3(u))]])
+                                     [dx1(dx2(u)), dx2(dx2(u)), dx2(dx3(u))],
+                                     [dx1(dx3(u)), dx2(dx3(u)), dx3(dx3(u))]])
 
 #==============================================================================
 
